@@ -2,12 +2,15 @@
 from __future__ import annotations
 
 from abc import abstractmethod
+import asyncio
 from collections.abc import Callable
 import json
 import logging
+from typing import Any, Protocol, cast, final
 
 import voluptuous as vol
 
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_CONFIGURATION_URL,
     ATTR_MANUFACTURER,
@@ -19,12 +22,19 @@ from homeassistant.const import (
     CONF_DEVICE,
     CONF_ENTITY_CATEGORY,
     CONF_ICON,
+    CONF_MODEL,
     CONF_NAME,
     CONF_UNIQUE_ID,
     CONF_VALUE_TEMPLATE,
 )
-from homeassistant.core import callback
-from homeassistant.helpers import config_validation as cv
+from homeassistant.core import CALLBACK_TYPE, Event, HomeAssistant, callback
+from homeassistant.helpers import (
+    config_validation as cv,
+    device_registry as dr,
+    discovery,
+    entity_registry as er,
+)
+from homeassistant.helpers.device_registry import EVENT_DEVICE_REGISTRY_UPDATED
 from homeassistant.helpers.dispatcher import (
     async_dispatcher_connect,
     async_dispatcher_send,
@@ -36,9 +46,11 @@ from homeassistant.helpers.entity import (
     EntityCategory,
     async_generate_entity_id,
 )
-from homeassistant.helpers.typing import ConfigType
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 
-from . import DATA_MQTT, MqttValueTemplate, debug_info, publish, subscription
+from . import debug_info, subscription
+from .client import async_publish
 from .const import (
     ATTR_DISCOVERY_HASH,
     ATTR_DISCOVERY_PAYLOAD,
@@ -47,13 +59,19 @@ from .const import (
     CONF_ENCODING,
     CONF_QOS,
     CONF_TOPIC,
+    DATA_MQTT,
+    DATA_MQTT_CONFIG,
+    DATA_MQTT_RELOAD_NEEDED,
+    DATA_MQTT_UPDATED_CONFIG,
+    DEFAULT_ENCODING,
     DEFAULT_PAYLOAD_AVAILABLE,
     DEFAULT_PAYLOAD_NOT_AVAILABLE,
     DOMAIN,
     MQTT_CONNECTED,
     MQTT_DISCONNECTED,
+    MQTT_RELOADED,
 )
-from .debug_info import log_messages
+from .debug_info import log_message, log_messages
 from .discovery import (
     MQTT_DISCOVERY_DONE,
     MQTT_DISCOVERY_NEW,
@@ -61,8 +79,12 @@ from .discovery import (
     clear_discovery_hash,
     set_discovery_hash,
 )
-from .models import ReceiveMessage
-from .subscription import async_subscribe_topics, async_unsubscribe_topics
+from .models import MqttValueTemplate, PublishPayloadType, ReceiveMessage
+from .subscription import (
+    async_prepare_subscribe_topics,
+    async_subscribe_topics,
+    async_unsubscribe_topics,
+)
 from .util import valid_subscribe_topic
 
 _LOGGER = logging.getLogger(__name__)
@@ -85,7 +107,6 @@ CONF_JSON_ATTRS_TEMPLATE = "json_attributes_template"
 CONF_IDENTIFIERS = "identifiers"
 CONF_CONNECTIONS = "connections"
 CONF_MANUFACTURER = "manufacturer"
-CONF_MODEL = "model"
 CONF_SW_VERSION = "sw_version"
 CONF_VIA_DEVICE = "via_device"
 CONF_DEPRECATED_VIA_HUB = "via_hub"
@@ -201,6 +222,91 @@ MQTT_ENTITY_COMMON_SCHEMA = MQTT_AVAILABILITY_SCHEMA.extend(
 )
 
 
+def warn_for_legacy_schema(domain: str) -> Callable:
+    """Warn once when a legacy platform schema is used."""
+    warned = set()
+
+    def validator(config: ConfigType) -> ConfigType:
+        """Return a validator."""
+        nonlocal warned
+
+        if domain in warned:
+            return config
+
+        _LOGGER.warning(
+            "Manually configured MQTT %s(s) found under platform key '%s', "
+            "please move to the mqtt integration key, see "
+            "https://www.home-assistant.io/integrations/%s.mqtt/#new_format",
+            domain,
+            domain,
+            domain,
+        )
+        warned.add(domain)
+        return config
+
+    return validator
+
+
+class SetupEntity(Protocol):
+    """Protocol type for async_setup_entities."""
+
+    async def __call__(
+        self,
+        hass: HomeAssistant,
+        async_add_entities: AddEntitiesCallback,
+        config: ConfigType,
+        config_entry: ConfigEntry | None = None,
+        discovery_data: dict[str, Any] | None = None,
+    ) -> None:
+        """Define setup_entities type."""
+
+
+async def async_setup_platform_discovery(
+    hass: HomeAssistant, platform_domain: str
+) -> CALLBACK_TYPE:
+    """Set up platform discovery for manual config."""
+
+    async def _async_discover_entities() -> None:
+        """Discover entities for a platform."""
+        if DATA_MQTT_UPDATED_CONFIG in hass.data:
+            # The platform has been reloaded
+            config_yaml = hass.data[DATA_MQTT_UPDATED_CONFIG]
+        else:
+            config_yaml = hass.data.get(DATA_MQTT_CONFIG, {})
+        if not config_yaml:
+            return
+        if platform_domain not in config_yaml:
+            return
+        await asyncio.gather(
+            *(
+                discovery.async_load_platform(hass, platform_domain, DOMAIN, config, {})
+                for config in await async_get_platform_config_from_yaml(
+                    hass, platform_domain, config_yaml
+                )
+            )
+        )
+
+    unsub = async_dispatcher_connect(hass, MQTT_RELOADED, _async_discover_entities)
+    await _async_discover_entities()
+    return unsub
+
+
+async def async_get_platform_config_from_yaml(
+    hass: HomeAssistant,
+    platform_domain: str,
+    config_yaml: ConfigType = None,
+) -> list[ConfigType]:
+    """Return a list of validated configurations for the domain."""
+
+    if config_yaml is None:
+        config_yaml = hass.data.get(DATA_MQTT_CONFIG)
+    if not config_yaml:
+        return []
+    if not (platform_configs := config_yaml.get(platform_domain)):
+        return []
+    return platform_configs
+
+
 async def async_setup_entry_helper(hass, domain, async_setup, schema):
     """Set up entity, automation or tag creation dynamically through MQTT discovery."""
 
@@ -223,6 +329,25 @@ async def async_setup_entry_helper(hass, domain, async_setup, schema):
     )
 
 
+async def async_setup_platform_helper(
+    hass: HomeAssistant,
+    platform_domain: str,
+    config: ConfigType | DiscoveryInfoType,
+    async_add_entities: AddEntitiesCallback,
+    async_setup_entities: SetupEntity,
+) -> None:
+    """Return true if platform setup should be aborted."""
+    if not bool(hass.config_entries.async_entries(DOMAIN)):
+        hass.data[DATA_MQTT_RELOAD_NEEDED] = None
+        _LOGGER.warning(
+            "MQTT integration is not setup, skipping setup of manually configured "
+            "MQTT %s",
+            platform_domain,
+        )
+        return
+    await async_setup_entities(hass, async_add_entities, config)
+
+
 def init_entity_id_from_config(hass, entity, config, entity_id_format):
     """Set entity_id from object_id if defined in config."""
     if CONF_OBJECT_ID in config:
@@ -245,14 +370,19 @@ class MqttAttributes(Entity):
     async def async_added_to_hass(self) -> None:
         """Subscribe MQTT events."""
         await super().async_added_to_hass()
+        self._attributes_prepare_subscribe_topics()
         await self._attributes_subscribe_topics()
 
+    def attributes_prepare_discovery_update(self, config: dict):
+        """Handle updated discovery message."""
+        self._attributes_config = config
+        self._attributes_prepare_subscribe_topics()
+
     async def attributes_discovery_update(self, config: dict):
         """Handle updated discovery message."""
-        self._attributes_config = config
         await self._attributes_subscribe_topics()
 
-    async def _attributes_subscribe_topics(self):
+    def _attributes_prepare_subscribe_topics(self):
         """(Re)Subscribe to topics."""
         attr_tpl = MqttValueTemplate(
             self._attributes_config.get(CONF_JSON_ATTRS_TEMPLATE), entity=self
@@ -280,7 +410,7 @@ class MqttAttributes(Entity):
                 _LOGGER.warning("Erroneous JSON: %s", payload)
                 self._attributes = None
 
-        self._attributes_sub_state = await async_subscribe_topics(
+        self._attributes_sub_state = async_prepare_subscribe_topics(
             self.hass,
             self._attributes_sub_state,
             {
@@ -293,9 +423,13 @@ class MqttAttributes(Entity):
             },
         )
 
+    async def _attributes_subscribe_topics(self):
+        """(Re)Subscribe to topics."""
+        await async_subscribe_topics(self.hass, self._attributes_sub_state)
+
     async def async_will_remove_from_hass(self):
         """Unsubscribe when removed."""
-        self._attributes_sub_state = await async_unsubscribe_topics(
+        self._attributes_sub_state = async_unsubscribe_topics(
             self.hass, self._attributes_sub_state
         )
 
@@ -318,6 +452,7 @@ class MqttAvailability(Entity):
     async def async_added_to_hass(self) -> None:
         """Subscribe MQTT events."""
         await super().async_added_to_hass()
+        self._availability_prepare_subscribe_topics()
         await self._availability_subscribe_topics()
         self.async_on_remove(
             async_dispatcher_connect(self.hass, MQTT_CONNECTED, self.async_mqtt_connect)
@@ -328,9 +463,13 @@ class MqttAvailability(Entity):
             )
         )
 
-    async def availability_discovery_update(self, config: dict):
+    def availability_prepare_discovery_update(self, config: dict):
         """Handle updated discovery message."""
         self._availability_setup_from_config(config)
+        self._availability_prepare_subscribe_topics()
+
+    async def availability_discovery_update(self, config: dict):
+        """Handle updated discovery message."""
         await self._availability_subscribe_topics()
 
     def _availability_setup_from_config(self, config):
@@ -351,10 +490,7 @@ class MqttAvailability(Entity):
                     CONF_AVAILABILITY_TEMPLATE: avail.get(CONF_VALUE_TEMPLATE),
                 }
 
-        for (
-            topic,  # pylint: disable=unused-variable
-            avail_topic_conf,
-        ) in self._avail_topics.items():
+        for avail_topic_conf in self._avail_topics.values():
             avail_topic_conf[CONF_AVAILABILITY_TEMPLATE] = MqttValueTemplate(
                 avail_topic_conf[CONF_AVAILABILITY_TEMPLATE],
                 entity=self,
@@ -362,7 +498,7 @@ class MqttAvailability(Entity):
 
         self._avail_config = config
 
-    async def _availability_subscribe_topics(self):
+    def _availability_prepare_subscribe_topics(self):
         """(Re)Subscribe to topics."""
 
         @callback
@@ -394,12 +530,16 @@ class MqttAvailability(Entity):
             for topic in self._avail_topics
         }
 
-        self._availability_sub_state = await async_subscribe_topics(
+        self._availability_sub_state = async_prepare_subscribe_topics(
             self.hass,
             self._availability_sub_state,
             topics,
         )
 
+    async def _availability_subscribe_topics(self):
+        """(Re)Subscribe to topics."""
+        await async_subscribe_topics(self.hass, self._availability_sub_state)
+
     @callback
     def async_mqtt_connect(self):
         """Update state on connection/disconnection to MQTT broker."""
@@ -408,7 +548,7 @@ class MqttAvailability(Entity):
 
     async def async_will_remove_from_hass(self):
         """Unsubscribe when removed."""
-        self._availability_sub_state = await async_unsubscribe_topics(
+        self._availability_sub_state = async_unsubscribe_topics(
             self.hass, self._availability_sub_state
         )
 
@@ -426,33 +566,187 @@ class MqttAvailability(Entity):
         return self._available_latest
 
 
-async def cleanup_device_registry(hass, device_id):
-    """Remove device registry entry if there are no remaining entities or triggers."""
+async def cleanup_device_registry(
+    hass: HomeAssistant, device_id: str | None, config_entry_id: str | None
+) -> None:
+    """Remove MQTT from the device registry entry if there are no remaining entities, triggers or tags."""
     # Local import to avoid circular dependencies
-    # pylint: disable=import-outside-toplevel
+    # pylint: disable-next=import-outside-toplevel
     from . import device_trigger, tag
 
-    device_registry = await hass.helpers.device_registry.async_get_registry()
-    entity_registry = await hass.helpers.entity_registry.async_get_registry()
+    device_registry = dr.async_get(hass)
+    entity_registry = er.async_get(hass)
     if (
         device_id
-        and not hass.helpers.entity_registry.async_entries_for_device(
+        and config_entry_id
+        and not er.async_entries_for_device(
             entity_registry, device_id, include_disabled_entities=False
         )
         and not await device_trigger.async_get_triggers(hass, device_id)
         and not tag.async_has_tags(hass, device_id)
     ):
-        device_registry.async_remove_device(device_id)
+        device_registry.async_update_device(
+            device_id, remove_config_entry_id=config_entry_id
+        )
+
+
+def get_discovery_hash(discovery_data: dict) -> tuple:
+    """Get the discovery hash from the discovery data."""
+    return discovery_data[ATTR_DISCOVERY_HASH]
+
+
+def send_discovery_done(hass: HomeAssistant, discovery_data: dict) -> None:
+    """Acknowledge a discovery message has been handled."""
+    discovery_hash = get_discovery_hash(discovery_data)
+    async_dispatcher_send(hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None)
+
+
+def stop_discovery_updates(
+    hass: HomeAssistant,
+    discovery_data: dict,
+    remove_discovery_updated: Callable[[], None] | None = None,
+) -> None:
+    """Stop discovery updates of being sent."""
+    if remove_discovery_updated:
+        remove_discovery_updated()
+        remove_discovery_updated = None
+    discovery_hash = get_discovery_hash(discovery_data)
+    clear_discovery_hash(hass, discovery_hash)
+
+
+async def async_remove_discovery_payload(hass: HomeAssistant, discovery_data: dict):
+    """Clear retained discovery topic in broker to avoid rediscovery after a restart of HA."""
+    discovery_topic = discovery_data[ATTR_DISCOVERY_TOPIC]
+    await async_publish(hass, discovery_topic, "", retain=True)
+
+
+class MqttDiscoveryDeviceUpdate:
+    """Add support for auto discovery for platforms without an entity."""
+
+    def __init__(
+        self,
+        hass: HomeAssistant,
+        discovery_data: dict,
+        device_id: str | None,
+        config_entry: ConfigEntry,
+        log_name: str,
+    ) -> None:
+        """Initialize the update service."""
+
+        self.hass = hass
+        self.log_name = log_name
+
+        self._discovery_data = discovery_data
+        self._device_id = device_id
+        self._config_entry = config_entry
+        self._config_entry_id = config_entry.entry_id
+        self._skip_device_removal: bool = False
+
+        discovery_hash = get_discovery_hash(discovery_data)
+        self._remove_discovery_updated = async_dispatcher_connect(
+            hass,
+            MQTT_DISCOVERY_UPDATED.format(discovery_hash),
+            self.async_discovery_update,
+        )
+        if device_id is not None:
+            self._remove_device_updated = hass.bus.async_listen(
+                EVENT_DEVICE_REGISTRY_UPDATED, self._async_device_removed
+            )
+        _LOGGER.info(
+            "%s %s has been initialized",
+            self.log_name,
+            discovery_hash,
+        )
+
+    async def async_discovery_update(
+        self,
+        discovery_payload: DiscoveryInfoType | None,
+    ) -> None:
+        """Handle discovery update."""
+        discovery_hash = get_discovery_hash(self._discovery_data)
+        _LOGGER.info(
+            "Got update for %s with hash: %s '%s'",
+            self.log_name,
+            discovery_hash,
+            discovery_payload,
+        )
+        if (
+            discovery_payload
+            and discovery_payload != self._discovery_data[ATTR_DISCOVERY_PAYLOAD]
+        ):
+            _LOGGER.info(
+                "%s %s updating",
+                self.log_name,
+                discovery_hash,
+            )
+            await self.async_update(discovery_payload)
+        if not discovery_payload:
+            # Unregister and clean up the current discovery instance
+            stop_discovery_updates(
+                self.hass, self._discovery_data, self._remove_discovery_updated
+            )
+            await self._async_tear_down()
+            send_discovery_done(self.hass, self._discovery_data)
+            _LOGGER.info(
+                "%s %s has been removed",
+                self.log_name,
+                discovery_hash,
+            )
+        else:
+            # Normal update without change
+            send_discovery_done(self.hass, self._discovery_data)
+            _LOGGER.info(
+                "%s %s no changes",
+                self.log_name,
+                discovery_hash,
+            )
+            return
+
+    async def _async_device_removed(self, event: Event) -> None:
+        """Handle the manual removal of a device."""
+        if self._skip_device_removal or not async_removed_from_device(
+            self.hass, event, cast(str, self._device_id), self._config_entry_id
+        ):
+            return
+        # Prevent a second cleanup round after the device is removed
+        self._remove_device_updated()
+        self._skip_device_removal = True
+        # Unregister and clean up and publish an empty payload
+        # so the service is not rediscovered after a restart
+        stop_discovery_updates(
+            self.hass, self._discovery_data, self._remove_discovery_updated
+        )
+        await self._async_tear_down()
+        await async_remove_discovery_payload(self.hass, self._discovery_data)
+
+    async def _async_tear_down(self) -> None:
+        """Handle the cleanup of the discovery service."""
+        # Cleanup platform resources
+        await self.async_tear_down()
+        # remove the service for auto discovery updates and clean up the device registry
+        if not self._skip_device_removal:
+            # Prevent a second cleanup round after the device is removed
+            self._skip_device_removal = True
+            await cleanup_device_registry(
+                self.hass, self._device_id, self._config_entry_id
+            )
+
+    async def async_update(self, discovery_data: dict) -> None:
+        """Handle the update of platform specific parts, extend to the platform."""
+
+    @abstractmethod
+    async def async_tear_down(self) -> None:
+        """Handle the cleanup of platform specific parts, extend to the platform."""
 
 
 class MqttDiscoveryUpdate(Entity):
-    """Mixin used to handle updated discovery message."""
+    """Mixin used to handle updated discovery message for entity based platforms."""
 
     def __init__(self, discovery_data, discovery_update=None) -> None:
         """Initialize the discovery update mixin."""
         self._discovery_data = discovery_data
         self._discovery_update = discovery_update
-        self._remove_signal: Callable | None = None
+        self._remove_discovery_updated: Callable | None = None
         self._removed_from_hass = False
 
     async def async_added_to_hass(self) -> None:
@@ -469,13 +763,12 @@ class MqttDiscoveryUpdate(Entity):
             Remove entity from entity registry if it is registered, this also removes the state.
             If the entity is not in the entity registry, just remove the state.
             """
-            entity_registry = (
-                await self.hass.helpers.entity_registry.async_get_registry()
-            )
-            if entity_registry.async_is_registered(self.entity_id):
-                entity_entry = entity_registry.async_get(self.entity_id)
+            entity_registry = er.async_get(self.hass)
+            if entity_entry := entity_registry.async_get(self.entity_id):
                 entity_registry.async_remove(self.entity_id)
-                await cleanup_device_registry(self.hass, entity_entry.device_id)
+                await cleanup_device_registry(
+                    self.hass, entity_entry.device_id, entity_entry.config_entry_id
+                )
             else:
                 await self.async_remove(force_remove=True)
 
@@ -501,9 +794,7 @@ class MqttDiscoveryUpdate(Entity):
                 else:
                     # Non-empty, unchanged payload: Ignore to avoid changing states
                     _LOGGER.info("Ignoring unchanged update for: %s", self.entity_id)
-            async_dispatcher_send(
-                self.hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None
-            )
+            send_discovery_done(self.hass, self._discovery_data)
 
         if discovery_hash:
             debug_info.add_entity_discovery_data(
@@ -511,14 +802,11 @@ class MqttDiscoveryUpdate(Entity):
             )
             # Set in case the entity has been removed and is re-added, for example when changing entity_id
             set_discovery_hash(self.hass, discovery_hash)
-            self._remove_signal = async_dispatcher_connect(
+            self._remove_discovery_updated = async_dispatcher_connect(
                 self.hass,
                 MQTT_DISCOVERY_UPDATED.format(discovery_hash),
                 discovery_callback,
             )
-            async_dispatcher_send(
-                self.hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None
-            )
 
     async def async_removed_from_registry(self) -> None:
         """Clear retained discovery topic in broker."""
@@ -528,18 +816,14 @@ class MqttDiscoveryUpdate(Entity):
             self._cleanup_discovery_on_remove()
 
             # Clear the discovery topic so the entity is not rediscovered after a restart
-            discovery_topic = self._discovery_data[ATTR_DISCOVERY_TOPIC]
-            publish(self.hass, discovery_topic, "", retain=True)
+            await async_remove_discovery_payload(self.hass, self._discovery_data)
 
     @callback
     def add_to_platform_abort(self) -> None:
         """Abort adding an entity to a platform."""
         if self._discovery_data:
-            discovery_hash = self._discovery_data[ATTR_DISCOVERY_HASH]
-            clear_discovery_hash(self.hass, discovery_hash)
-            async_dispatcher_send(
-                self.hass, MQTT_DISCOVERY_DONE.format(discovery_hash), None
-            )
+            stop_discovery_updates(self.hass, self._discovery_data)
+            send_discovery_done(self.hass, self._discovery_data)
         super().add_to_platform_abort()
 
     async def async_will_remove_from_hass(self) -> None:
@@ -549,14 +833,11 @@ class MqttDiscoveryUpdate(Entity):
     def _cleanup_discovery_on_remove(self) -> None:
         """Stop listening to signal and cleanup discovery data."""
         if self._discovery_data and not self._removed_from_hass:
-            debug_info.remove_entity_data(self.hass, self.entity_id)
-            clear_discovery_hash(self.hass, self._discovery_data[ATTR_DISCOVERY_HASH])
+            stop_discovery_updates(
+                self.hass, self._discovery_data, self._remove_discovery_updated
+            )
             self._removed_from_hass = True
 
-        if self._remove_signal:
-            self._remove_signal()
-            self._remove_signal = None
-
 
 def device_info_from_config(config) -> DeviceInfo | None:
     """Return a device description for device registry."""
@@ -600,10 +881,10 @@ class MqttEntityDeviceInfo(Entity):
         self._device_config = device_config
         self._config_entry = config_entry
 
-    async def device_info_discovery_update(self, config: dict):
+    def device_info_discovery_update(self, config: dict):
         """Handle updated discovery message."""
         self._device_config = config.get(CONF_DEVICE)
-        device_registry = await self.hass.helpers.device_registry.async_get_registry()
+        device_registry = dr.async_get(self.hass)
         config_entry_id = self._config_entry.entry_id
         device_info = self.device_info
 
@@ -653,30 +934,68 @@ class MqttEntity(
             self.hass, self, self._config, self._entity_id_format
         )
 
+    @final
     async def async_added_to_hass(self):
-        """Subscribe mqtt events."""
+        """Subscribe to MQTT events."""
         await super().async_added_to_hass()
+        self._prepare_subscribe_topics()
         await self._subscribe_topics()
+        await self.mqtt_async_added_to_hass()
+        if self._discovery_data is not None:
+            send_discovery_done(self.hass, self._discovery_data)
+
+    async def mqtt_async_added_to_hass(self):
+        """Call before the discovery message is acknowledged.
+
+        To be extended by subclasses.
+        """
 
     async def discovery_update(self, discovery_payload):
         """Handle updated discovery message."""
         config = self.config_schema()(discovery_payload)
         self._config = config
         self._setup_from_config(self._config)
+
+        # Prepare MQTT subscriptions
+        self.attributes_prepare_discovery_update(config)
+        self.availability_prepare_discovery_update(config)
+        self.device_info_discovery_update(config)
+        self._prepare_subscribe_topics()
+
+        # Finalize MQTT subscriptions
         await self.attributes_discovery_update(config)
         await self.availability_discovery_update(config)
-        await self.device_info_discovery_update(config)
         await self._subscribe_topics()
         self.async_write_ha_state()
 
     async def async_will_remove_from_hass(self):
         """Unsubscribe when removed."""
-        self._sub_state = await subscription.async_unsubscribe_topics(
+        self._sub_state = subscription.async_unsubscribe_topics(
             self.hass, self._sub_state
         )
         await MqttAttributes.async_will_remove_from_hass(self)
         await MqttAvailability.async_will_remove_from_hass(self)
         await MqttDiscoveryUpdate.async_will_remove_from_hass(self)
+        debug_info.remove_entity_data(self.hass, self.entity_id)
+
+    async def async_publish(
+        self,
+        topic: str,
+        payload: PublishPayloadType,
+        qos: int = 0,
+        retain: bool = False,
+        encoding: str = DEFAULT_ENCODING,
+    ):
+        """Publish message to an MQTT topic."""
+        log_message(self.hass, self.entity_id, topic, payload, qos, retain)
+        await async_publish(
+            self.hass,
+            topic,
+            payload,
+            qos,
+            retain,
+            encoding,
+        )
 
     @staticmethod
     @abstractmethod
@@ -686,6 +1005,10 @@ class MqttEntity(
     def _setup_from_config(self, config):
         """(Re)Setup the entity."""
 
+    @abstractmethod
+    def _prepare_subscribe_topics(self):
+        """(Re)Subscribe to topics."""
+
     @abstractmethod
     async def _subscribe_topics(self):
         """(Re)Subscribe to topics."""
@@ -696,7 +1019,7 @@ class MqttEntity(
         return self._config[CONF_ENABLED_BY_DEFAULT]
 
     @property
-    def entity_category(self) -> EntityCategory | str | None:
+    def entity_category(self) -> EntityCategory | None:
         """Return the entity category if any."""
         return self._config.get(CONF_ENTITY_CATEGORY)
 
@@ -719,3 +1042,51 @@ class MqttEntity(
     def unique_id(self):
         """Return a unique ID."""
         return self._unique_id
+
+
+def update_device(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    config: ConfigType,
+) -> str | None:
+    """Update device registry."""
+    if CONF_DEVICE not in config:
+        return None
+
+    device = None
+    device_registry = dr.async_get(hass)
+    config_entry_id = config_entry.entry_id
+    device_info = device_info_from_config(config[CONF_DEVICE])
+
+    if config_entry_id is not None and device_info is not None:
+        update_device_info = cast(dict, device_info)
+        update_device_info["config_entry_id"] = config_entry_id
+        device = device_registry.async_get_or_create(**update_device_info)
+
+    return device.id if device else None
+
+
+@callback
+def async_removed_from_device(
+    hass: HomeAssistant, event: Event, mqtt_device_id: str, config_entry_id: str
+) -> bool:
+    """Check if the passed event indicates MQTT was removed from a device."""
+    device_id = event.data["device_id"]
+    if event.data["action"] not in ("remove", "update"):
+        return False
+
+    if device_id != mqtt_device_id:
+        return False
+
+    if event.data["action"] == "update":
+        if "config_entries" not in event.data["changes"]:
+            return False
+        device_registry = dr.async_get(hass)
+        if not (device_entry := device_registry.async_get(device_id)):
+            # The device is already removed, do cleanup when we get "remove" event
+            return False
+        if config_entry_id in device_entry.config_entries:
+            # Not removed from device
+            return False
+
+    return True
