@@ -1,58 +1,59 @@
 """SQLAlchemy util functions."""
 from __future__ import annotations
 
-from collections.abc import Callable, Generator
+from collections.abc import Callable, Generator, Iterable
 from contextlib import contextmanager
 from datetime import date, datetime, timedelta
 import functools
 import logging
 import os
 import time
-from typing import TYPE_CHECKING, Any
+from typing import TYPE_CHECKING, Any, TypeVar
 
 from awesomeversion import (
     AwesomeVersion,
     AwesomeVersionException,
     AwesomeVersionStrategy,
 )
+import ciso8601
 from sqlalchemy import text
 from sqlalchemy.engine.cursor import CursorFetchStrategy
+from sqlalchemy.engine.row import Row
 from sqlalchemy.exc import OperationalError, SQLAlchemyError
 from sqlalchemy.orm.query import Query
 from sqlalchemy.orm.session import Session
+from typing_extensions import Concatenate, ParamSpec
 
 from homeassistant.core import HomeAssistant
 import homeassistant.util.dt as dt_util
 
-from .const import DATA_INSTANCE, SQLITE_URL_PREFIX
+from .const import DATA_INSTANCE, SQLITE_URL_PREFIX, SupportedDialect
 from .models import (
-    ALL_TABLES,
     TABLE_RECORDER_RUNS,
     TABLE_SCHEMA_CHANGES,
-    TABLE_STATISTICS,
-    TABLE_STATISTICS_META,
-    TABLE_STATISTICS_RUNS,
-    TABLE_STATISTICS_SHORT_TERM,
+    TABLES_TO_CHECK,
     RecorderRuns,
+    UnsupportedDialect,
     process_timestamp,
 )
 
 if TYPE_CHECKING:
     from . import Recorder
 
+_RecorderT = TypeVar("_RecorderT", bound="Recorder")
+_P = ParamSpec("_P")
+
 _LOGGER = logging.getLogger(__name__)
 
 RETRIES = 3
 QUERY_RETRY_WAIT = 0.1
 SQLITE3_POSTFIXES = ["", "-wal", "-shm"]
+DEFAULT_YIELD_STATES_ROWS = 32768
 
 MIN_VERSION_MARIA_DB = AwesomeVersion("10.3.0", AwesomeVersionStrategy.SIMPLEVER)
-MIN_VERSION_MARIA_DB_ROWNUM = AwesomeVersion("10.2.0", AwesomeVersionStrategy.SIMPLEVER)
 MIN_VERSION_MYSQL = AwesomeVersion("8.0.0", AwesomeVersionStrategy.SIMPLEVER)
-MIN_VERSION_MYSQL_ROWNUM = AwesomeVersion("5.8.0", AwesomeVersionStrategy.SIMPLEVER)
 MIN_VERSION_PGSQL = AwesomeVersion("12.0", AwesomeVersionStrategy.SIMPLEVER)
 MIN_VERSION_SQLITE = AwesomeVersion("3.31.0", AwesomeVersionStrategy.SIMPLEVER)
-MIN_VERSION_SQLITE_ROWNUM = AwesomeVersion("3.25.0", AwesomeVersionStrategy.SIMPLEVER)
 
 # This is the maximum time after the recorder ends the session
 # before we no longer consider startup to be a "restart" and we
@@ -119,7 +120,7 @@ def commit(session: Session, work: Any) -> bool:
 
 def execute(
     qry: Query, to_native: bool = False, validate_entity_ids: bool = True
-) -> list:
+) -> list[Row]:
     """Query the database and convert the objects to HA native form.
 
     This method also retries a few times in the case of stale connections.
@@ -162,7 +163,40 @@ def execute(
                 raise
             time.sleep(QUERY_RETRY_WAIT)
 
-    assert False  # unreachable
+    assert False  # unreachable # pragma: no cover
+
+
+def execute_stmt(
+    session: Session,
+    query: Query,
+    start_time: datetime | None = None,
+    end_time: datetime | None = None,
+    yield_per: int | None = DEFAULT_YIELD_STATES_ROWS,
+) -> Iterable[Row]:
+    """Execute a StatementLambdaElement.
+
+    If the time window passed is greater than one day
+    the execution method will switch to yield_per to
+    reduce memory pressure.
+
+    It is not recommended to pass a time window
+    when selecting non-ranged rows (ie selecting
+    specific entities) since they are usually faster
+    with .all().
+    """
+    use_all = not start_time or ((end_time or dt_util.utcnow()) - start_time).days <= 1
+    for tryno in range(0, RETRIES):
+        try:
+            if use_all:
+                return session.execute(query).all()  # type: ignore[no-any-return]
+            return session.execute(query).yield_per(yield_per)  # type: ignore[no-any-return]
+        except SQLAlchemyError as err:
+            _LOGGER.error("Error executing query: %s", err)
+            if tryno == RETRIES - 1:
+                raise
+            time.sleep(QUERY_RETRY_WAIT)
+
+    assert False  # unreachable # pragma: no cover
 
 
 def validate_or_move_away_sqlite_database(dburl: str) -> bool:
@@ -209,15 +243,7 @@ def last_run_was_recently_clean(cursor: CursorFetchStrategy) -> bool:
 def basic_sanity_check(cursor: CursorFetchStrategy) -> bool:
     """Check tables to make sure select does not fail."""
 
-    for table in ALL_TABLES:
-        # The statistics tables may not be present in old databases
-        if table in [
-            TABLE_STATISTICS,
-            TABLE_STATISTICS_META,
-            TABLE_STATISTICS_RUNS,
-            TABLE_STATISTICS_SHORT_TERM,
-        ]:
-            continue
+    for table in TABLES_TO_CHECK:
         if table in (TABLE_RECORDER_RUNS, TABLE_SCHEMA_CHANGES):
             cursor.execute(f"SELECT * FROM {table};")  # nosec # not injection
         else:
@@ -300,29 +326,31 @@ def query_on_connection(dbapi_connection: Any, statement: str) -> Any:
     return result
 
 
-def _warn_unsupported_dialect(dialect_name: str) -> None:
+def _fail_unsupported_dialect(dialect_name: str) -> None:
     """Warn about unsupported database version."""
-    _LOGGER.warning(
+    _LOGGER.error(
         "Database %s is not supported; Home Assistant supports %s. "
-        "Starting with Home Assistant 2022.2 this will prevent the recorder from "
-        "starting. Please migrate your database to a supported software before then",
+        "Starting with Home Assistant 2022.6 this prevents the recorder from "
+        "starting. Please migrate your database to a supported software",
         dialect_name,
         "MariaDB ≥ 10.3, MySQL ≥ 8.0, PostgreSQL ≥ 12, SQLite ≥ 3.31.0",
     )
+    raise UnsupportedDialect
 
 
-def _warn_unsupported_version(
+def _fail_unsupported_version(
     server_version: str, dialect_name: str, minimum_version: str
 ) -> None:
     """Warn about unsupported database version."""
-    _LOGGER.warning(
+    _LOGGER.error(
         "Version %s of %s is not supported; minimum supported version is %s. "
-        "Starting with Home Assistant 2022.2 this will prevent the recorder from "
-        "starting. Please upgrade your database software before then",
+        "Starting with Home Assistant 2022.6 this prevents the recorder from "
+        "starting. Please upgrade your database software",
         server_version,
         dialect_name,
         minimum_version,
     )
+    raise UnsupportedDialect
 
 
 def _extract_version_from_server_response(
@@ -339,17 +367,39 @@ def _extract_version_from_server_response(
         return None
 
 
+def _datetime_or_none(value: str) -> datetime | None:
+    """Fast version of mysqldb DateTime_or_None.
+
+    https://github.com/PyMySQL/mysqlclient/blob/v2.1.0/MySQLdb/times.py#L66
+    """
+    try:
+        return ciso8601.parse_datetime(value)
+    except ValueError:
+        return None
+
+
+def build_mysqldb_conv() -> dict:
+    """Build a MySQLDB conv dict that uses cisco8601 to parse datetimes."""
+    # Late imports since we only call this if they are using mysqldb
+    from MySQLdb.constants import (  # pylint: disable=import-outside-toplevel,import-error
+        FIELD_TYPE,
+    )
+    from MySQLdb.converters import (  # pylint: disable=import-outside-toplevel,import-error
+        conversions,
+    )
+
+    return {**conversions, FIELD_TYPE.DATETIME: _datetime_or_none}
+
+
 def setup_connection_for_dialect(
     instance: Recorder,
     dialect_name: str,
     dbapi_connection: Any,
     first_connection: bool,
-) -> None:
+) -> AwesomeVersion | None:
     """Execute statements needed for dialect connection."""
-    # Returns False if the the connection needs to be setup
-    # on the next connection, returns True if the connection
-    # never needs to be setup again.
-    if dialect_name == "sqlite":
+    version: AwesomeVersion | None = None
+    if dialect_name == SupportedDialect.SQLITE:
         if first_connection:
             old_isolation = dbapi_connection.isolation_level
             dbapi_connection.isolation_level = None
@@ -362,22 +412,28 @@ def setup_connection_for_dialect(
             version_string = result[0][0]
             version = _extract_version_from_server_response(version_string)
 
-            if version and version < MIN_VERSION_SQLITE_ROWNUM:
-                instance._db_supports_row_number = (  # pylint: disable=[protected-access]
-                    False
-                )
             if not version or version < MIN_VERSION_SQLITE:
-                _warn_unsupported_version(
+                _fail_unsupported_version(
                     version or version_string, "SQLite", MIN_VERSION_SQLITE
                 )
 
-        # approximately 8MiB of memory
-        execute_on_connection(dbapi_connection, "PRAGMA cache_size = -8192")
+        # The upper bound on the cache size is approximately 16MiB of memory
+        execute_on_connection(dbapi_connection, "PRAGMA cache_size = -16384")
+
+        #
+        # Enable FULL synchronous if they have a commit interval of 0
+        # or NORMAL if they do not.
+        #
+        # https://sqlite.org/pragma.html#pragma_synchronous
+        # The synchronous=NORMAL setting is a good choice for most applications running in WAL mode.
+        #
+        synchronous = "NORMAL" if instance.commit_interval else "FULL"
+        execute_on_connection(dbapi_connection, f"PRAGMA synchronous={synchronous}")
 
         # enable support for foreign keys
         execute_on_connection(dbapi_connection, "PRAGMA foreign_keys=ON")
 
-    elif dialect_name == "mysql":
+    elif dialect_name == SupportedDialect.MYSQL:
         execute_on_connection(dbapi_connection, "SET session wait_timeout=28800")
         if first_connection:
             result = query_on_connection(dbapi_connection, "SELECT VERSION()")
@@ -386,37 +442,31 @@ def setup_connection_for_dialect(
             is_maria_db = "mariadb" in version_string.lower()
 
             if is_maria_db:
-                if version and version < MIN_VERSION_MARIA_DB_ROWNUM:
-                    instance._db_supports_row_number = (  # pylint: disable=[protected-access]
-                        False
-                    )
                 if not version or version < MIN_VERSION_MARIA_DB:
-                    _warn_unsupported_version(
+                    _fail_unsupported_version(
                         version or version_string, "MariaDB", MIN_VERSION_MARIA_DB
                     )
             else:
-                if version and version < MIN_VERSION_MYSQL_ROWNUM:
-                    instance._db_supports_row_number = (  # pylint: disable=[protected-access]
-                        False
-                    )
                 if not version or version < MIN_VERSION_MYSQL:
-                    _warn_unsupported_version(
+                    _fail_unsupported_version(
                         version or version_string, "MySQL", MIN_VERSION_MYSQL
                     )
 
-    elif dialect_name == "postgresql":
+    elif dialect_name == SupportedDialect.POSTGRESQL:
         if first_connection:
             # server_version_num was added in 2006
             result = query_on_connection(dbapi_connection, "SHOW server_version")
             version_string = result[0][0]
             version = _extract_version_from_server_response(version_string)
             if not version or version < MIN_VERSION_PGSQL:
-                _warn_unsupported_version(
+                _fail_unsupported_version(
                     version or version_string, "PostgreSQL", MIN_VERSION_PGSQL
                 )
 
     else:
-        _warn_unsupported_dialect(dialect_name)
+        _fail_unsupported_dialect(dialect_name)
+
+    return version
 
 
 def end_incomplete_runs(session: Session, start_time: datetime) -> None:
@@ -430,21 +480,28 @@ def end_incomplete_runs(session: Session, start_time: datetime) -> None:
         session.add(run)
 
 
-def retryable_database_job(description: str) -> Callable:
+def retryable_database_job(
+    description: str,
+) -> Callable[
+    [Callable[Concatenate[_RecorderT, _P], bool]],
+    Callable[Concatenate[_RecorderT, _P], bool],
+]:
     """Try to execute a database job.
 
     The job should return True if it finished, and False if it needs to be rescheduled.
     """
 
-    def decorator(job: Callable[[Any], bool]) -> Callable:
+    def decorator(
+        job: Callable[Concatenate[_RecorderT, _P], bool]
+    ) -> Callable[Concatenate[_RecorderT, _P], bool]:
         @functools.wraps(job)
-        def wrapper(instance: Recorder, *args: Any, **kwargs: Any) -> bool:
+        def wrapper(instance: _RecorderT, *args: _P.args, **kwargs: _P.kwargs) -> bool:
             try:
                 return job(instance, *args, **kwargs)
             except OperationalError as err:
                 assert instance.engine is not None
                 if (
-                    instance.engine.dialect.name == "mysql"
+                    instance.engine.dialect.name == SupportedDialect.MYSQL
                     and err.orig.args[0] in RETRYABLE_MYSQL_ERRORS
                 ):
                     _LOGGER.info(
@@ -470,7 +527,7 @@ def periodic_db_cleanups(instance: Recorder) -> None:
     These cleanups will happen nightly or after any purge.
     """
     assert instance.engine is not None
-    if instance.engine.dialect.name == "sqlite":
+    if instance.engine.dialect.name == SupportedDialect.SQLITE:
         # Execute sqlite to create a wal checkpoint and free up disk space
         _LOGGER.debug("WAL checkpoint")
         with instance.engine.connect() as connection:
