@@ -39,7 +39,7 @@ from .const import (
     DOMAIN,
     LOGGER,
 )
-from .device import Device, async_get_mac_address_from_host
+from .device import Device, async_create_device, async_get_mac_address_from_host
 
 NOTIFICATION_ID = "upnp_notification"
 NOTIFICATION_TITLE = "UPnP/IGD Setup"
@@ -113,8 +113,7 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     try:
         await asyncio.wait_for(device_discovered_event.wait(), timeout=10)
     except asyncio.TimeoutError as err:
-        LOGGER.debug("Device not discovered: %s", usn)
-        raise ConfigEntryNotReady from err
+        raise ConfigEntryNotReady(f"Device not discovered: {usn}") from err
     finally:
         cancel_discovered_callback()
 
@@ -123,12 +122,11 @@ async def async_setup_entry(hass: HomeAssistant, entry: ConfigEntry) -> bool:
     assert discovery_info.ssdp_location is not None
     location = discovery_info.ssdp_location
     try:
-        device = await Device.async_create_device(hass, location)
+        device = await async_create_device(hass, location)
     except UpnpConnectionError as err:
-        LOGGER.debug(
-            "Error connecting to device at location: %s, err: %s", location, err
-        )
-        raise ConfigEntryNotReady from err
+        raise ConfigEntryNotReady(
+            f"Error connecting to device at location: {location}, err: {err}"
+        ) from err
 
     # Track the original UDN such that existing sensors do not change their unique_id.
     if CONFIG_ENTRY_ORIGINAL_UDN not in entry.data:
@@ -255,21 +253,15 @@ class UpnpDataUpdateCoordinator(DataUpdateCoordinator):
             LOGGER,
             name=device.name,
             update_interval=update_interval,
-            update_method=self._async_fetch_data,
         )
 
-    async def _async_fetch_data(self) -> Mapping[str, Any]:
+    async def _async_update_data(self) -> Mapping[str, Any]:
         """Update data."""
         try:
             update_values = await asyncio.gather(
                 self.device.async_get_traffic_data(),
                 self.device.async_get_status(),
             )
-
-            return {
-                **update_values[0],
-                **update_values[1],
-            }
         except UpnpCommunicationError as exception:
             LOGGER.debug(
                 "Caught exception when updating device: %s, exception: %s",
@@ -280,6 +272,11 @@ class UpnpDataUpdateCoordinator(DataUpdateCoordinator):
                 f"Unable to communicate with IGD at: {self.device.device_url}"
             ) from exception
 
+        return {
+            **update_values[0],
+            **update_values[1],
+        }
+
 
 class UpnpEntity(CoordinatorEntity[UpnpDataUpdateCoordinator]):
     """Base class for UPnP/IGD entities."""
